#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#  Author: wh0ami
# License: MIT License <https://opensource.org/license/MIT>
# Project: https://codeberg.org/wh0ami/tibber-pulse-sml-debugger


class AuthenticationError(Exception):
    pass
